@@ -1,6 +1,10 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
 
+#include "multipagewidget.h"
+#include "multipagewidgetplugin.h"
+#include "multipagewidgetextensionfactory.h"
+
 #include <QtDesigner/QExtensionFactory>
 #include <QtDesigner/QExtensionManager>
 #include <QtDesigner/QDesignerFormEditorInterface>
@@ -11,9 +15,7 @@
 #include <QIcon>
 #include <QtPlugin>
 
-#include "multipagewidget.h"
-#include "multipagewidgetplugin.h"
-#include "multipagewidgetextensionfactory.h"
+using namespace Qt::StringLiterals;
 
 //! [0]
 MultiPageWidgetPlugin::MultiPageWidgetPlugin(QObject *parent)
@@ -23,32 +25,32 @@ MultiPageWidgetPlugin::MultiPageWidgetPlugin(QObject *parent)
 
 QString MultiPageWidgetPlugin::name() const
 {
-    return QLatin1String("MultiPageWidget");
+    return u"MultiPageWidget"_s;
 }
 
 QString MultiPageWidgetPlugin::group() const
 {
-    return QLatin1String("Display Widgets [Examples]");
+    return u"Display Widgets [Examples]"_s;
 }
 
 QString MultiPageWidgetPlugin::toolTip() const
 {
-    return QString();
+    return {};
 }
 
 QString MultiPageWidgetPlugin::whatsThis() const
 {
-    return QString();
+    return {};
 }
 
 QString MultiPageWidgetPlugin::includeFile() const
 {
-    return QLatin1String("multipagewidget.h");
+    return u"multipagewidget.h"_s;
 }
 
 QIcon MultiPageWidgetPlugin::icon() const
 {
-    return QIcon();
+    return {};
 }
 
 //! [0] //! [1]
@@ -60,7 +62,7 @@ bool MultiPageWidgetPlugin::isContainer() const
 //! [1] //! [2]
 QWidget *MultiPageWidgetPlugin::createWidget(QWidget *parent)
 {
-    MultiPageWidget *widget = new MultiPageWidget(parent);
+    auto *widget = new MultiPageWidget(parent);
     connect(widget, &MultiPageWidget::currentIndexChanged,
             this, &MultiPageWidgetPlugin::currentIndexChanged);
     connect(widget, &MultiPageWidget::pageTitleChanged,
@@ -83,11 +85,11 @@ void MultiPageWidgetPlugin::initialize(QDesignerFormEditorInterface *formEditor)
 //! [4]
 
 //! [5]
-    QExtensionManager *manager = formEditor->extensionManager();
+    auto *manager = formEditor->extensionManager();
 //! [5] //! [6]
-    QExtensionFactory *factory = new MultiPageWidgetExtensionFactory(manager);
+    auto *factory = new MultiPageWidgetExtensionFactory(manager);
 
-    Q_ASSERT(manager != 0);
+    Q_ASSERT(manager != nullptr);
     manager->registerExtensions(factory, Q_TYPEID(QDesignerContainerExtension));
 
     initialized = true;
@@ -97,10 +99,10 @@ void MultiPageWidgetPlugin::initialize(QDesignerFormEditorInterface *formEditor)
 //! [7]
 QString MultiPageWidgetPlugin::domXml() const
 {
-    return QLatin1String(R"(
+    return uR"(
 <ui language="c++">
     <widget class="MultiPageWidget" name="multipagewidget">
-        <widget class="QWidget" name="page" />
+        <widget class="QWidget" name="page"/>
     </widget>
     <customwidgets>
         <customwidget>
@@ -109,7 +111,7 @@ QString MultiPageWidgetPlugin::domXml() const
             <addpagemethod>addPage</addpagemethod>
         </customwidget>
     </customwidgets>
-</ui>)");
+</ui>)"_s;
 }
 //! [7]
 
@@ -117,10 +119,10 @@ QString MultiPageWidgetPlugin::domXml() const
 void MultiPageWidgetPlugin::currentIndexChanged(int index)
 {
     Q_UNUSED(index);
-    MultiPageWidget *widget = qobject_cast<MultiPageWidget*>(sender());
+    auto *widget = qobject_cast<MultiPageWidget*>(sender());
 //! [8] //! [9]
     if (widget) {
-        QDesignerFormWindowInterface *form = QDesignerFormWindowInterface::findFormWindow(widget);
+        auto *form = QDesignerFormWindowInterface::findFormWindow(widget);
         if (form)
             form->emitSelectionChanged();
     }
@@ -131,20 +133,18 @@ void MultiPageWidgetPlugin::currentIndexChanged(int index)
 void MultiPageWidgetPlugin::pageTitleChanged(const QString &title)
 {
     Q_UNUSED(title);
-    MultiPageWidget *widget = qobject_cast<MultiPageWidget*>(sender());
+    auto *widget = qobject_cast<MultiPageWidget*>(sender());
 //! [10] //! [11]
     if (widget) {
-        QWidget *page = widget->widget(widget->currentIndex());
-        QDesignerFormWindowInterface *form;
-        form = QDesignerFormWindowInterface::findFormWindow(widget);
+        auto *page = widget->widget(widget->currentIndex());
+        auto *form = QDesignerFormWindowInterface::findFormWindow(widget);
 //! [11]
         if (form) {
 //! [12]
-            QDesignerFormEditorInterface *editor = form->core();
-            QExtensionManager *manager = editor->extensionManager();
+            auto *editor = form->core();
+            auto *manager = editor->extensionManager();
 //! [12] //! [13]
-            QDesignerPropertySheetExtension *sheet;
-            sheet = qt_extension<QDesignerPropertySheetExtension*>(manager, page);
+            auto *sheet = qt_extension<QDesignerPropertySheetExtension*>(manager, page);
             const int propertyIndex = sheet->indexOf(QLatin1String("windowTitle"));
             sheet->setChanged(propertyIndex, true);
         }
