@@ -3,8 +3,9 @@
 
 #include "paletteeditor.h"
 
+#include <qdesigner_utils_p.h>
 #include <iconloader_p.h>
-#include <qtcolorbutton.h>
+#include <qtcolorbutton_p.h>
 
 #include <private/formbuilderextra_p.h>
 #include <private/ui4_p.h>
@@ -19,6 +20,7 @@
 #include <QtWidgets/qlabel.h>
 #include <QtWidgets/qmenu.h>
 #include <QtWidgets/qheaderview.h>
+#include <QtWidgets/qstyle.h>
 
 #include <QtGui/qaction.h>
 #if QT_CONFIG(clipboard)
@@ -35,6 +37,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 namespace qdesigner_internal {
 
 enum { BrushRole = 33 };
@@ -50,7 +54,21 @@ PaletteEditor::PaletteEditor(QDesignerFormEditorInterface *core, QWidget *parent
     auto loadButton = ui.buttonBox->addButton(tr("Load..."), QDialogButtonBox::ActionRole);
     connect(loadButton, &QPushButton::clicked, this, &PaletteEditor::load);
 
+    connect(ui.buildButton, &QtColorButton::colorChanged,
+            this, &PaletteEditor::buildButtonColorChanged);
+    connect(ui.activeRadio, &QAbstractButton::clicked,
+            this, &PaletteEditor::activeRadioClicked);
+    connect(ui.inactiveRadio,  &QAbstractButton::clicked,
+            this, &PaletteEditor::inactiveRadioClicked);
+    connect(ui.disabledRadio, &QAbstractButton::clicked,
+            this, &PaletteEditor::disabledRadioClicked);
+    connect(ui.computeRadio, &QAbstractButton::clicked,
+            this, &PaletteEditor::computeRadioClicked);
+    connect(ui.detailsRadio, &QAbstractButton::clicked,
+            this, &PaletteEditor::detailsRadioClicked);
+
     ui.paletteView->setModel(m_paletteModel);
+    ui.previewGroupBox->setTitle(tr("Preview (%1)").arg(style()->objectName()));
     updatePreviewPalette();
     updateStyledButton();
     ui.paletteView->setModel(m_paletteModel);
@@ -85,18 +103,15 @@ QPalette PaletteEditor::palette() const
 void PaletteEditor::setPalette(const QPalette &palette)
 {
     m_editPalette = palette;
-    const uint mask = palette.resolveMask();
-    for (int i = 0; i < static_cast<int>(QPalette::NColorRoles); ++i) {
-        if (!(mask & (1 << i))) {
-            m_editPalette.setBrush(QPalette::Active, static_cast<QPalette::ColorRole>(i),
-                        m_parentPalette.brush(QPalette::Active, static_cast<QPalette::ColorRole>(i)));
-            m_editPalette.setBrush(QPalette::Inactive, static_cast<QPalette::ColorRole>(i),
-                        m_parentPalette.brush(QPalette::Inactive, static_cast<QPalette::ColorRole>(i)));
-            m_editPalette.setBrush(QPalette::Disabled, static_cast<QPalette::ColorRole>(i),
-                        m_parentPalette.brush(QPalette::Disabled, static_cast<QPalette::ColorRole>(i)));
+    for (int r = 0; r < static_cast<int>(QPalette::NColorRoles); ++r) {
+        for (int g = 0; g < static_cast<int>(QPalette::NColorGroups); ++g) {
+            const auto role = static_cast<QPalette::ColorRole>(r);
+            const auto group = static_cast<QPalette::ColorGroup>(g);
+            if (!palette.isBrushSet(group, role))
+                m_editPalette.setBrush(group, role, m_parentPalette.brush(group, role));
         }
     }
-    m_editPalette.setResolveMask(mask);
+    m_editPalette.setResolveMask(palette.resolveMask());
     updatePreviewPalette();
     updateStyledButton();
     m_paletteUpdated = true;
@@ -111,30 +126,30 @@ void PaletteEditor::setPalette(const QPalette &palette, const QPalette &parentPa
     setPalette(palette);
 }
 
-void PaletteEditor::on_buildButton_colorChanged(const QColor &)
+void PaletteEditor::buildButtonColorChanged()
 {
     buildPalette();
 }
 
-void PaletteEditor::on_activeRadio_clicked()
+void PaletteEditor::activeRadioClicked()
 {
     m_currentColorGroup = QPalette::Active;
     updatePreviewPalette();
 }
 
-void PaletteEditor::on_inactiveRadio_clicked()
+void PaletteEditor::inactiveRadioClicked()
 {
     m_currentColorGroup = QPalette::Inactive;
     updatePreviewPalette();
 }
 
-void PaletteEditor::on_disabledRadio_clicked()
+void PaletteEditor::disabledRadioClicked()
 {
     m_currentColorGroup = QPalette::Disabled;
     updatePreviewPalette();
 }
 
-void PaletteEditor::on_computeRadio_clicked()
+void PaletteEditor::computeRadioClicked()
 {
     if (m_compute)
         return;
@@ -144,7 +159,7 @@ void PaletteEditor::on_computeRadio_clicked()
     m_paletteModel->setCompute(true);
 }
 
-void PaletteEditor::on_detailsRadio_clicked()
+void PaletteEditor::detailsRadioClicked()
 {
     if (!m_compute)
         return;
@@ -204,15 +219,12 @@ QPalette PaletteEditor::getPalette(QDesignerFormEditorInterface *core, QWidget*
 {
     PaletteEditor dlg(core, parent);
     QPalette parentPalette(parentPal);
-    uint mask = init.resolveMask();
-    for (int i = 0; i < static_cast<int>(QPalette::NColorRoles); ++i) {
-        if (!(mask & (1 << i))) {
-            parentPalette.setBrush(QPalette::Active, static_cast<QPalette::ColorRole>(i),
-                        init.brush(QPalette::Active, static_cast<QPalette::ColorRole>(i)));
-            parentPalette.setBrush(QPalette::Inactive, static_cast<QPalette::ColorRole>(i),
-                        init.brush(QPalette::Inactive, static_cast<QPalette::ColorRole>(i)));
-            parentPalette.setBrush(QPalette::Disabled, static_cast<QPalette::ColorRole>(i),
-                        init.brush(QPalette::Disabled, static_cast<QPalette::ColorRole>(i)));
+    for (int r = 0; r < static_cast<int>(QPalette::NColorRoles); ++r) {
+        for (int g = 0; g < static_cast<int>(QPalette::NColorGroups); ++g) {
+            const auto role = static_cast<QPalette::ColorRole>(r);
+            const auto group = static_cast<QPalette::ColorGroup>(g);
+            if (!init.isBrushSet(group, role))
+                parentPalette.setBrush(group, role, init.brush(group, role));
         }
     }
     dlg.setPalette(init, parentPalette);
@@ -223,7 +235,7 @@ QPalette PaletteEditor::getPalette(QDesignerFormEditorInterface *core, QWidget*
     return result == QDialog::Accepted ? dlg.palette() : init;
 }
 
-void PaletteEditor::viewContextMenuRequested(const QPoint &pos)
+void PaletteEditor::viewContextMenuRequested(QPoint pos)
 {
     const auto index = ui.paletteView->indexAt(pos);
     if (!index.isValid())
@@ -260,8 +272,6 @@ void PaletteEditor::viewContextMenuRequested(const QPoint &pos)
     m_paletteModel->setData(index, QVariant(brush), BrushRole);
 }
 
-static inline QString paletteSuffix() { return QStringLiteral("xml"); }
-
 static inline QString paletteFilter()
 {
     return PaletteEditor::tr("QPalette UI file (*.xml)");
@@ -318,7 +328,7 @@ static bool loadPalette(const QString &fileName, QPalette *pal, QString *errorMe
         *errorMessage = msgCannotReadPalette(fileName, reader);
         return false;
     }
-    if (reader.name() != QLatin1String("palette")) {
+    if (reader.name() != "palette"_L1) {
         const auto why = PaletteEditor::tr("Invalid element \"%1\", expected \"palette\".")
                          .arg(reader.name().toString());
         *errorMessage = msgCannotReadPalette(fileName, reader, why);
@@ -338,7 +348,7 @@ void PaletteEditor::save()
 {
     QFileDialog dialog(this, tr("Save Palette"), QString(), paletteFilter());
     dialog.setAcceptMode(QFileDialog::AcceptSave);
-    dialog.setDefaultSuffix(paletteSuffix());
+    dialog.setDefaultSuffix(u"xml"_s);
     while (dialog.exec() == QDialog::Accepted) {
         QString errorMessage;
         if (savePalette(dialog.selectedFiles().constFirst(), palette(), &errorMessage))
@@ -363,6 +373,11 @@ void PaletteEditor::load()
 }
 
 //////////////////////
+// Column 0: Role name and reset button. Uses a boolean value indicating
+//           whether the role is modified for the edit role.
+// Column 1: Color group Active
+// Column 2: Color group Inactive (visibility depending on m_compute/detail radio group)
+// Column 3: Color group Disabled
 
 PaletteModel::PaletteModel(QObject *parent)  :
     QAbstractTableModel(parent)
@@ -375,7 +390,7 @@ PaletteModel::PaletteModel(QObject *parent)  :
     for (int r = QPalette::WindowText; r < QPalette::NColorRoles; r++) {
         const auto role = static_cast<QPalette::ColorRole>(r);
         if (role != QPalette::NoRole)
-            m_roleEntries.append({QLatin1String(e.key(r)), role});
+            m_roleEntries.append({QLatin1StringView(e.key(r)), role});
     }
 }
 
@@ -394,6 +409,12 @@ QBrush PaletteModel::brushAt(const QModelIndex &index) const
     return m_palette.brush(columnToGroup(index.column()), roleAt(index.row()));
 }
 
+// Palette resolve mask with all group bits for a row/role
+quint64 PaletteModel::rowMask(const QModelIndex &index) const
+{
+    return paletteResolveMask(roleAt(index.row()));
+}
+
 QVariant PaletteModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid())
@@ -403,15 +424,11 @@ QVariant PaletteModel::data(const QModelIndex &index, int role) const
     if (index.column() < 0 || index.column() >= 4)
         return QVariant();
 
-    if (index.column() == 0) {
+    if (index.column() == 0) { // Role name/bold print if changed
         if (role == Qt::DisplayRole)
             return m_roleEntries.at(index.row()).name;
-        if (role == Qt::EditRole) {
-            const uint mask = m_palette.resolveMask();
-            if (mask & (1 << int(roleAt(index.row()))))
-                return true;
-            return false;
-        }
+        if (role == Qt::EditRole)
+            return (rowMask(index) & m_palette.resolveMask()) != 0;
         return QVariant();
     }
     if (role == Qt::ToolTipRole)
@@ -470,11 +487,12 @@ bool PaletteModel::setData(const QModelIndex &index, const QVariant &value, int
         return true;
     }
     if (index.column() == 0 && role == Qt::EditRole) {
-        uint mask = m_palette.resolveMask();
+        auto mask = m_palette.resolveMask();
         const bool isMask = qvariant_cast<bool>(value);
-        if (isMask)
-            mask |= (1 << int(colorRole));
-        else {
+        const auto bitMask = rowMask(index);
+        if (isMask) {
+            mask |= bitMask;
+        } else {
             m_palette.setBrush(QPalette::Active, colorRole,
                                m_parentPalette.brush(QPalette::Active, colorRole));
             m_palette.setBrush(QPalette::Inactive, colorRole,
@@ -482,7 +500,7 @@ bool PaletteModel::setData(const QModelIndex &index, const QVariant &value, int
             m_palette.setBrush(QPalette::Disabled, colorRole,
                                m_parentPalette.brush(QPalette::Disabled, colorRole));
 
-            mask &= ~(1 << int(colorRole));
+            mask &= ~bitMask;
         }
         m_palette.setResolveMask(mask);
         emit paletteChanged(m_palette);
@@ -550,7 +568,7 @@ int PaletteModel::groupToColumn(QPalette::ColorGroup group) const
 
 int PaletteModel::rowOf(QPalette::ColorRole role) const
 {
-    for (int row = 0, size = m_roleEntries.size(); row < size; ++row) {
+    for (qsizetype row = 0, size = m_roleEntries.size(); row < size; ++row) {
         if (m_roleEntries.at(row).role == role)
             return row;
     }
@@ -610,7 +628,7 @@ RoleEditor::RoleEditor(QWidget *parent) :
 
     QToolButton *button = new QToolButton(this);
     button->setToolButtonStyle(Qt::ToolButtonIconOnly);
-    button->setIcon(createIconSet(QStringLiteral("resetproperty.png")));
+    button->setIcon(createIconSet("resetproperty.png"_L1));
     button->setIconSize(QSize(8,8));
     button->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::MinimumExpanding));
     layout->addWidget(button);
@@ -644,7 +662,7 @@ void RoleEditor::emitResetProperty()
 
 //////////////////////////
 ColorDelegate::ColorDelegate(QDesignerFormEditorInterface *core, QObject *parent) :
-    QItemDelegate(parent),
+    QStyledItemDelegate(parent),
     m_core(core)
 {
 }
@@ -704,7 +722,7 @@ void ColorDelegate::setModelData(QWidget *ed, QAbstractItemModel *model,
 void ColorDelegate::updateEditorGeometry(QWidget *ed,
                 const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-    QItemDelegate::updateEditorGeometry(ed, option, index);
+    QStyledItemDelegate::updateEditorGeometry(ed, option, index);
     ed->setGeometry(ed->geometry().adjusted(0, 0, -1, -1));
 }
 
@@ -734,7 +752,7 @@ void ColorDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt,
         painter->fillRect(option.rect, br);
         painter->restore();
     }
-    QItemDelegate::paint(painter, option, index);
+    QStyledItemDelegate::paint(painter, option, index);
 
 
     const QColor color = static_cast<QRgb>(QApplication::style()->styleHint(QStyle::SH_Table_GridLineColor, &option));
@@ -750,7 +768,7 @@ void ColorDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt,
 
 QSize ColorDelegate::sizeHint(const QStyleOptionViewItem &opt, const QModelIndex &index) const
 {
-    return QItemDelegate::sizeHint(opt, index) + QSize(4, 4);
+    return QStyledItemDelegate::sizeHint(opt, index) + QSize(4, 4);
 }
 }
 
