@@ -1,36 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Linguist of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "cpp.h"
 
 #include <translator.h>
 #include <QtCore/QBitArray>
-#include <QtCore/QStack>
 #include <QtCore/QTextStream>
 #include <QtCore/QRegularExpression>
 
@@ -39,11 +13,9 @@ QT_BEGIN_NAMESPACE
 
 /* qmake ignore Q_OBJECT */
 
-static QString MagicComment(QLatin1String("TRANSLATOR"));
+using namespace Qt::StringLiterals;
 
-#define STRING(s) static QString str##s(QLatin1String(#s))
-
-//#define DIAGNOSE_RETRANSLATABILITY // FIXME: should make a runtime option of this
+static const QString CppMagicComment = u"TRANSLATOR"_s;
 
 size_t qHash(const HashString &str)
 {
@@ -84,7 +56,7 @@ private:
     QBitArray m_ba;
 };
 
-class CppParser {
+class CppParser : private CppParserState {
 
 public:
     CppParser(ParseResults *results = 0);
@@ -96,14 +68,6 @@ public:
     const ParseResults *recordResults(bool isHeader);
     void deleteResults() { delete results; }
 
-    struct SavedState {
-        NamespaceList namespaces;
-        QStack<int> namespaceDepths;
-        NamespaceList functionContext;
-        QString functionContextUnresolved;
-        QString pendingContext;
-    };
-
 private:
     struct IfdefState {
         IfdefState() {}
@@ -114,7 +78,7 @@ private:
             elseLine(-1)
         {}
 
-        SavedState state;
+        CppParserState state;
         int bracketDepth, bracketDepth1st;
         int braceDepth, braceDepth1st;
         int parenDepth, parenDepth1st;
@@ -122,14 +86,13 @@ private:
     };
 
     enum TokenType {
-        Tok_Eof, Tok_class, Tok_friend, Tok_namespace, Tok_using, Tok_return,
-        Tok_Q_OBJECT, Tok_Access, Tok_Cancel,
+        Tok_Eof, Tok_class, Tok_enum, Tok_friend, Tok_namespace, Tok_using, Tok_return,
+        Tok_decltype, Tok_Q_OBJECT, Tok_Access, Tok_Cancel,
         Tok_Ident, Tok_String, Tok_RawString, Tok_Arrow, Tok_Colon, Tok_ColonColon,
-        Tok_Equals, Tok_LeftBracket, Tok_RightBracket, Tok_QuestionMark,
+        Tok_Equals, Tok_LeftBracket, Tok_RightBracket, Tok_AngleBracket, Tok_QuestionMark,
         Tok_LeftBrace, Tok_RightBrace, Tok_LeftParen, Tok_RightParen, Tok_Comma, Tok_Semicolon,
         Tok_Null, Tok_Integer,
-        Tok_QuotedInclude, Tok_AngledInclude,
-        Tok_Other
+        Tok_QuotedInclude, Tok_AngledInclude
     };
 
     std::ostream &yyMsg(int line = 0);
@@ -159,8 +122,8 @@ private:
     void processInclude(const QString &file, ConversionData &cd,
                         const QStringList &includeStack, QSet<QString> &inclusions);
 
-    void saveState(SavedState *state);
-    void loadState(const SavedState *state);
+    void saveState(CppParserState *state);
+    void loadState(const CppParserState &state);
 
     static QString stringifyNamespace(int start, const NamespaceList &namespaces);
     static QString stringifyNamespace(const NamespaceList &namespaces)
@@ -226,17 +189,12 @@ private:
     QString sourcetext;
     TranslatorMessage::ExtraData extra;
 
-    NamespaceList namespaces;
-    QStack<int> namespaceDepths;
-    NamespaceList functionContext;
-    QString functionContextUnresolved;
     QString prospectiveContext;
-    QString pendingContext;
     ParseResults *results;
     Translator *tor;
     bool directInclude;
 
-    SavedState savedState;
+    CppParserState savedState;
     int yyMinBraceDepth;
     bool inDefine;
 };
@@ -364,25 +322,44 @@ CppParser::TokenType CppParser::lookAheadToSemicolonOrLeftBrace()
     }
 }
 
-STRING(Q_OBJECT);
-STRING(class);
-STRING(final);
-STRING(friend);
-STRING(namespace);
-STRING(nullptr);
-STRING(Q_NULLPTR);
-STRING(NULL);
-STRING(operator);
-STRING(return);
-STRING(struct);
-STRING(using);
-STRING(private);
-STRING(protected);
-STRING(public);
-STRING(slots);
-STRING(signals);
-STRING(Q_SLOTS);
-STRING(Q_SIGNALS);
+static bool isStringLiteralPrefix(const QStringView s)
+{
+    return s == u"L"_s
+        || s == u"U"_s
+        || s == u"u"_s
+        || s == u"u8"_s;
+}
+
+static bool isRawStringLiteralPrefix(QStringView s)
+{
+    if (s.endsWith(u'R')) {
+        s.chop(1);
+        return s.isEmpty() || isStringLiteralPrefix(s);
+    }
+    return false;
+}
+
+static const QString strQ_OBJECT = u"Q_OBJECT"_s;
+static const QString strclass = u"class"_s;
+static const QString strdecltype = u"decltype"_s;
+static const QString strenum = u"enum"_s;
+static const QString strfinal = u"final"_s;
+static const QString strfriend = u"friend"_s;
+static const QString strnamespace = u"namespace"_s;
+static const QString strnullptr = u"nullptr"_s;
+static const QString strQ_NULLPTR = u"Q_NULLPTR"_s;
+static const QString strNULL = u"NULL"_s;
+static const QString stroperator = u"operator"_s;
+static const QString strreturn = u"return"_s;
+static const QString strstruct = u"struct"_s;
+static const QString strusing = u"using"_s;
+static const QString strprivate = u"private"_s;
+static const QString strprotected = u"protected"_s;
+static const QString strpublic = u"public"_s;
+static const QString strslots = u"slots"_s;
+static const QString strsignals = u"signals"_s;
+static const QString strQ_SLOTS = u"Q_SLOTS"_s;
+static const QString strQ_SIGNALS = u"Q_SIGNALS"_s;
 
 CppParser::TokenType CppParser::getToken()
 {
@@ -540,7 +517,7 @@ CppParser::TokenType CppParser::getToken()
                             yyBracketDepth = is.bracketDepth1st;
                             yyBraceDepth = is.braceDepth1st;
                             yyParenDepth = is.parenDepth1st;
-                            loadState(&is.state);
+                            loadState(is.state);
                         }
                     }
                     yyCh = getChar();
@@ -592,6 +569,11 @@ CppParser::TokenType CppParser::getToken()
 
             //qDebug() << "IDENT: " << yyWord;
 
+            if (yyCh == '"' && isStringLiteralPrefix(yyWord)) {
+                // Handle prefixed string literals as ordinary string literals.
+                continue;
+            }
+
             switch (yyWord.unicode()[0].unicode()) {
             case 'N':
                 if (yyWord == strNULL)
@@ -609,6 +591,14 @@ CppParser::TokenType CppParser::getToken()
                 if (yyWord == strclass)
                     return Tok_class;
                 break;
+            case 'd':
+                if (yyWord == strdecltype)
+                    return Tok_decltype;
+                break;
+            case 'e':
+                if (yyWord == strenum)
+                    return Tok_enum;
+                break;
             case 'f':
                 if (yyWord == strfriend)
                     return Tok_friend;
@@ -654,10 +644,7 @@ CppParser::TokenType CppParser::getToken()
             }
 
             // a C++11 raw string literal?
-            if (yyCh == '"' && (
-                        yyWord == QLatin1String("R") || yyWord == QLatin1String("LR") || yyWord == QLatin1String("u8R") ||
-                        yyWord == QLatin1String("uR") || yyWord == QLatin1String("UR")
-                        )) {
+            if (yyCh == '"' && isRawStringLiteralPrefix(yyWord)) {
                 ptr = reinterpret_cast<ushort *>(const_cast<QChar *>(yyWord.unicode()));
                 //get delimiter
                 QString delimiter;
@@ -712,7 +699,7 @@ CppParser::TokenType CppParser::getToken()
             switch (yyCh) {
             case '\n':
                 if (inDefine) {
-                    loadState(&savedState);
+                    loadState(savedState);
                     prospectiveContext.clear();
                     yyBraceDepth = yyMinBraceDepth;
                     yyMinBraceDepth = 0;
@@ -805,7 +792,7 @@ CppParser::TokenType CppParser::getToken()
             case '>':
             case '<':
                 yyCh = getChar();
-                return Tok_Other;
+                return Tok_AngleBracket;
             case '\'':
                 yyCh = getChar();
                 if (yyCh == '\\')
@@ -883,7 +870,7 @@ CppParser::TokenType CppParser::getToken()
                 return Tok_QuestionMark;
             case '0':
                 yyCh = getChar();
-                if (yyCh == 'x') {
+                if (yyCh == 'x' || yyCh == 'X') {
                     do {
                         yyCh = getChar();
                     } while ((yyCh >= '0' && yyCh <= '9') || yyCh == '\''
@@ -920,38 +907,30 @@ CppParser::TokenType CppParser::getToken()
   utilities for the third part.
 */
 
-void CppParser::saveState(SavedState *state)
+void CppParser::saveState(CppParserState *state)
 {
-    state->namespaces = namespaces;
-    state->namespaceDepths = namespaceDepths;
-    state->functionContext = functionContext;
-    state->functionContextUnresolved = functionContextUnresolved;
-    state->pendingContext = pendingContext;
+    *state = *this;
 }
 
-void CppParser::loadState(const SavedState *state)
+void CppParser::loadState(const CppParserState &state)
 {
-    namespaces = state->namespaces;
-    namespaceDepths = state->namespaceDepths;
-    functionContext = state->functionContext;
-    functionContextUnresolved = state->functionContextUnresolved;
-    pendingContext = state->pendingContext;
+    *static_cast<CppParserState *>(this) = state;
 }
 
 Namespace *CppParser::modifyNamespace(NamespaceList *namespaces, bool haveLast)
 {
     Namespace *pns, *ns = &results->rootNamespace;
-    for (int i = 1; i < namespaces->count(); ++i) {
+    for (int i = 1; i < namespaces->size(); ++i) {
         pns = ns;
         if (!(ns = pns->children.value(namespaces->at(i)))) {
             do {
                 ns = new Namespace;
-                if (haveLast || i < namespaces->count() - 1)
+                if (haveLast || i < namespaces->size() - 1)
                     if (const Namespace *ons = findNamespace(*namespaces, i + 1))
                         ns->classDef = ons->classDef;
                 pns->children.insert(namespaces->at(i), ns);
                 pns = ns;
-            } while (++i < namespaces->count());
+            } while (++i < namespaces->size());
             break;
         }
     }
@@ -962,10 +941,10 @@ QString CppParser::stringifyNamespace(int start, const NamespaceList &namespaces
 {
     QString ret;
     int l = 0;
-    for (int j = start; j < namespaces.count(); ++j)
-        l += namespaces.at(j).value().length();
-    ret.reserve(l + qMax(0, (namespaces.count() - start - 1)) * 2);
-    for (int i = start; i < namespaces.count(); ++i) {
+    for (int j = start; j < namespaces.size(); ++j)
+        l += namespaces.at(j).value().size();
+    ret.reserve(l + qMax(0, (namespaces.size() - start - 1)) * 2);
+    for (int i = start; i < namespaces.size(); ++i) {
         if (i > start)
             ret += QLatin1String("::");
         ret += namespaces.at(i).value();
@@ -1049,7 +1028,7 @@ bool CppParser::qualifyOneCallbackUsing(const Namespace *ns, void *context) cons
     for (const HashStringList &use : ns->usings)
         if (!data->visitedUsings->contains(use)) {
             data->visitedUsings->insert(use);
-            if (qualifyOne(use.value(), use.value().count(), data->segment, data->resolved,
+            if (qualifyOne(use.value(), use.value().size(), data->segment, data->resolved,
                            data->visitedUsings))
                 return true;
         }
@@ -1084,7 +1063,7 @@ bool CppParser::fullyQualify(const NamespaceList &namespaces, int nsCnt,
 
     if (segments.first().value().isEmpty()) {
         // fully qualified
-        if (segments.count() == 1) {
+        if (segments.size() == 1) {
             resolved->clear();
             *resolved << HashString(QString());
             return true;
@@ -1099,8 +1078,8 @@ bool CppParser::fullyQualify(const NamespaceList &namespaces, int nsCnt,
     do {
         if (qualifyOne(namespaces, nsIdx + 1, segments[initSegIdx], resolved)) {
             int segIdx = initSegIdx;
-            while (++segIdx < segments.count()) {
-                if (!qualifyOne(*resolved, resolved->count(), segments[segIdx], resolved)) {
+            while (++segIdx < segments.size()) {
+                if (!qualifyOne(*resolved, resolved->size(), segments[segIdx], resolved)) {
                     if (unresolved)
                         *unresolved = segments.mid(segIdx);
                     return false;
@@ -1120,7 +1099,7 @@ bool CppParser::fullyQualify(const NamespaceList &namespaces,
                              const NamespaceList &segments, bool isDeclaration,
                              NamespaceList *resolved, NamespaceList *unresolved) const
 {
-    return fullyQualify(namespaces, namespaces.count(),
+    return fullyQualify(namespaces, namespaces.size(),
                         segments, isDeclaration, resolved, unresolved);
 }
 
@@ -1146,7 +1125,7 @@ const Namespace *CppParser::findNamespace(const NamespaceList &namespaces, int n
 {
     const Namespace *ns = 0;
     if (nsCount == -1)
-        nsCount = namespaces.count();
+        nsCount = namespaces.size();
     visitNamespace(namespaces, nsCount, &CppParser::findNamespaceCallback, &ns);
     return ns;
 }
@@ -1160,14 +1139,32 @@ void CppParser::enterNamespace(NamespaceList *namespaces, const HashString &name
 
 void CppParser::truncateNamespaces(NamespaceList *namespaces, int length)
 {
-    if (namespaces->count() > length)
+    if (namespaces->size() > length)
         namespaces->erase(namespaces->begin() + length, namespaces->end());
 }
 
+
 /*
   Functions for processing include files.
 */
 
+size_t qHash(const CppParserState &s, size_t seed)
+{
+    seed = qHash(s.namespaces, seed);
+    seed = qHash(s.namespaceDepths, seed);
+    seed = qHash(s.functionContext, seed);
+    seed = qHash(s.functionContextUnresolved, seed);
+    seed = qHash(s.pendingContext, seed);
+    return seed;
+}
+
+size_t qHash(const ResultsCacheKey &key, size_t seed)
+{
+    seed = qHash(key.cleanFile, seed);
+    seed = qHash(key.parserState, seed);
+    return seed;
+}
+
 IncludeCycleHash &CppFiles::includeCycles()
 {
     static IncludeCycleHash cycles;
@@ -1189,9 +1186,9 @@ QSet<QString> &CppFiles::blacklistedFiles()
     return blacklisted;
 }
 
-QSet<const ParseResults *> CppFiles::getResults(const QString &cleanFile)
+QSet<const ParseResults *> CppFiles::getResults(const ResultsCacheKey &key)
 {
-    IncludeCycle * const cycle = includeCycles().value(cleanFile);
+    IncludeCycle * const cycle = includeCycles().value(key);
 
     if (cycle)
         return cycle->results;
@@ -1199,16 +1196,16 @@ QSet<const ParseResults *> CppFiles::getResults(const QString &cleanFile)
         return QSet<const ParseResults *>();
 }
 
-void CppFiles::setResults(const QString &cleanFile, const ParseResults *results)
+void CppFiles::setResults(const ResultsCacheKey &key, const ParseResults *results)
 {
-    IncludeCycle *cycle = includeCycles().value(cleanFile);
+    IncludeCycle *cycle = includeCycles().value(key);
 
     if (!cycle) {
         cycle = new IncludeCycle;
-        includeCycles().insert(cleanFile, cycle);
+        includeCycles().insert(key, cycle);
     }
 
-    cycle->fileNames.insert(cleanFile);
+    cycle->fileNames.insert(key.cleanFile);
     cycle->results.insert(results);
 }
 
@@ -1232,14 +1229,15 @@ void CppFiles::setBlacklisted(const QString &cleanFile)
     blacklistedFiles().insert(cleanFile);
 }
 
-void CppFiles::addIncludeCycle(const QSet<QString> &fileNames)
+void CppFiles::addIncludeCycle(const QSet<QString> &fileNames, const CppParserState &parserState)
 {
     IncludeCycle * const cycle = new IncludeCycle;
     cycle->fileNames = fileNames;
 
     QSet<IncludeCycle *> intersectingCycles;
     for (const QString &fileName : fileNames) {
-        IncludeCycle *intersectingCycle = includeCycles().value(fileName);
+        const ResultsCacheKey key = { fileName, parserState };
+        IncludeCycle *intersectingCycle = includeCycles().value(key);
 
         if (intersectingCycle && !intersectingCycles.contains(intersectingCycle)) {
             intersectingCycles.insert(intersectingCycle);
@@ -1250,8 +1248,8 @@ void CppFiles::addIncludeCycle(const QSet<QString> &fileNames)
     }
     qDeleteAll(intersectingCycles);
 
-    for (const QString &fileName : qAsConst(cycle->fileNames))
-        includeCycles().insert(fileName, cycle);
+    for (const QString &fileName : std::as_const(cycle->fileNames))
+        includeCycles().insert({ fileName, parserState }, cycle);
 }
 
 static bool isHeader(const QString &name)
@@ -1265,29 +1263,27 @@ void CppParser::processInclude(const QString &file, ConversionData &cd, const QS
 {
     QString cleanFile = QDir::cleanPath(file);
 
-    for (const QString &ex : qAsConst(cd.m_excludes)) {
-        QRegularExpression rx(QRegularExpression::wildcardToRegularExpression(ex));
+    for (const QRegularExpression &rx : std::as_const(cd.m_excludes)) {
         if (rx.match(cleanFile).hasMatch())
             return;
     }
 
     const int index = includeStack.indexOf(cleanFile);
     if (index != -1) {
-        CppFiles::addIncludeCycle(QSet<QString>(includeStack.cbegin() + index, includeStack.cend()));
+        CppFiles::addIncludeCycle(QSet<QString>(includeStack.cbegin() + index, includeStack.cend()),
+                                  *this);
         return;
     }
 
-    // If the #include is in any kind of namespace, has been blacklisted previously,
+    // If the #include has been blacklisted previously,
     // or is not a header file (stdc++ extensionless or *.h*), then really include
     // it. Otherwise it is safe to process it stand-alone and re-use the parsed
     // namespace data for inclusion into other files.
     bool isIndirect = false;
-    if (namespaces.count() == 1 && functionContext.count() == 1
-        && functionContextUnresolved.isEmpty() && pendingContext.isEmpty()
-        && !CppFiles::isBlacklisted(cleanFile)
+    if (!CppFiles::isBlacklisted(cleanFile)
         && isHeader(cleanFile)) {
 
-        QSet<const ParseResults *> res = CppFiles::getResults(cleanFile);
+        QSet<const ParseResults *> res = CppFiles::getResults(ResultsCacheKey(cleanFile, *this));
         if (!res.isEmpty()) {
             results->includes.unite(res);
             return;
@@ -1310,7 +1306,7 @@ void CppParser::processInclude(const QString &file, ConversionData &cd, const QS
     inclusions.insert(cleanFile);
     if (isIndirect) {
         CppParser parser;
-        for (const QString &projectRoot : qAsConst(cd.m_projectRoots))
+        for (const QString &projectRoot : std::as_const(cd.m_projectRoots))
             if (cleanFile.startsWith(projectRoot)) {
                 parser.setTranslator(new Translator);
                 break;
@@ -1374,12 +1370,12 @@ bool CppParser::matchString(QString *s)
     }
 }
 
-STRING(QApplication);
-STRING(QCoreApplication);
-STRING(UnicodeUTF8);
-STRING(DefaultCodec);
-STRING(CodecForTr);
-STRING(Latin1);
+static const QString strQApplication = u"QApplication"_s;
+static const QString strQCoreApplication = u"QCoreApplication"_s;
+static const QString strUnicodeUTF8 = u"UnicodeUTF8"_s;
+static const QString strDefaultCodec = u"DefaultCodec"_s;
+static const QString strCodecForTr = u"CodecForTr"_s;
+static const QString strLatin1 = u"Latin1"_s;
 
 bool CppParser::matchEncoding()
 {
@@ -1490,7 +1486,7 @@ void CppParser::handleTr(QString &prefix, bool plural)
         }
         if (prefix.isEmpty()) {
             if (functionContextUnresolved.isEmpty()) {
-                int idx = functionContext.length();
+                int idx = functionContext.size();
                 if (idx < 2) {
                     yyMsg() << "tr() cannot be called without context\n";
                     return;
@@ -1525,14 +1521,6 @@ void CppParser::handleTr(QString &prefix, bool plural)
                 context = joinNamespaces(stringifyNamespace(functionContext), functionContextUnresolved);
             }
         } else {
-#ifdef DIAGNOSE_RETRANSLATABILITY
-            int last = prefix.lastIndexOf(QLatin1String("::"));
-            QString className = prefix.mid(last == -1 ? 0 : last + 2);
-            if (!className.isEmpty() && className == functionName) {
-                yyMsg() << qPrintable(QStringLiteral("It is not recommended to call tr() from within a constructor '%1::%2'\n")
-                        .arg(className).arg(functionName));
-            }
-#endif
             prefix.chop(2);
             NamespaceList nsl;
             NamespaceList unresolved;
@@ -1673,11 +1661,9 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
     static QString strColons(QLatin1String("::"));
 
     QString prefix;
-#ifdef DIAGNOSE_RETRANSLATABILITY
-    QString functionName;
-#endif
     bool yyTokColonSeen = false; // Start of c'tor's initializer list
     bool yyTokIdentSeen = false; // Start of initializer (member or base class)
+    bool maybeInTrailingReturnType = false;
     metaExpected = true;
 
     prospectiveContext.clear();
@@ -1693,7 +1679,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
         // so they don't confuse our scoping of static initializers.
         // we enter the loop by either reading a left bracket or by an
         // #else popping the state.
-        if (yyBracketDepth && yyBraceDepth == namespaceDepths.count()) {
+        if (yyBracketDepth && yyBraceDepth == namespaceDepths.size()) {
             yyTok = getToken();
             continue;
         }
@@ -1716,7 +1702,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                     processInclude(cSource, cd, includeStack, inclusions);
                 goto incOk;
             }
-            for (const QString &incPath : qAsConst(cd.m_includePath)) {
+            for (const QString &incPath : std::as_const(cd.m_includePath)) {
                 text = QDir(incPath).absoluteFilePath(yyWord);
                 text.detach();
                 if (QFileInfo(text).isFile()) {
@@ -1739,7 +1725,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
               Partial support for inlined functions.
             */
             yyTok = getToken();
-            if (yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0) {
+            if (yyBraceDepth == namespaceDepths.size() && yyParenDepth == 0) {
                 NamespaceList quali;
                 HashString fct;
 
@@ -1769,7 +1755,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                     }
                 }
 
-                if (yyTok == Tok_Colon || yyTok == Tok_Other) {
+                if (yyTok == Tok_Colon || yyTok == Tok_AngleBracket) {
                     // Skip any token until '{' or ';' since we might do things wrong if we find
                     // a '::' or ':' token here.
                     do {
@@ -1794,10 +1780,10 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                         yyMsg() << "Ignoring definition of undeclared qualified class\n";
                         break;
                     }
-                    namespaceDepths.push(namespaces.count());
+                    namespaceDepths.push(namespaces.size());
                     namespaces = nsl;
                 } else {
-                    namespaceDepths.push(namespaces.count());
+                    namespaceDepths.push(namespaces.size());
                 }
                 enterNamespace(&namespaces, fct);
 
@@ -1830,7 +1816,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                     ns = HashString(text);
                 }
                 if (yyTok == Tok_LeftBrace) {
-                    namespaceDepths.push(namespaces.count());
+                    namespaceDepths.push(namespaces.size());
                     for (const auto &nns : nestedNamespaces)
                         enterNamespace(&namespaces, nns);
                     enterNamespace(&namespaces, ns);
@@ -1863,7 +1849,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                 }
             } else if (yyTok == Tok_LeftBrace) {
                 // Anonymous namespace
-                namespaceDepths.push(namespaces.count());
+                namespaceDepths.push(namespaces.size());
                 metaExpected = true;
                 yyTok = getToken();
             }
@@ -1916,7 +1902,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             break;
         case Tok_Ident:
             if (yyTokColonSeen &&
-                yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0) {
+                yyBraceDepth == namespaceDepths.size() && yyParenDepth == 0) {
                 // member or base class identifier
                 yyTokIdentSeen = true;
             }
@@ -1964,18 +1950,18 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                 yyTok = getToken();
                 break;
             }
-            if (yyTok == Tok_ColonColon) {
+            if (yyTok == Tok_ColonColon && !maybeInTrailingReturnType) {
                 prefix += yyWord;
                 prefix.detach();
             } else {
               notrfunc:
                 prefix.clear();
-                if (yyTok == Tok_Ident && !yyParenDepth)
-                    prospectiveContext.clear();
             }
             metaExpected = false;
             break;
         case Tok_Arrow:
+            if (yyParenDepth == 0 && yyBraceDepth == namespaceDepths.size())
+                maybeInTrailingReturnType = true;
             yyTok = getToken();
             if (yyTok == Tok_Ident) {
                 switch (trFunctionAliasManager.trFunctionByName(yyWord)) {
@@ -1987,29 +1973,23 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             }
             break;
         case Tok_ColonColon:
-            if (yyTokIdentSeen) {
+            if (yyTokIdentSeen || maybeInTrailingReturnType) {
                 // member or base class identifier
                 yyTok = getToken();
                 break;
             }
-            if (yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0 && !yyTokColonSeen)
+            if (yyBraceDepth == namespaceDepths.size() && yyParenDepth == 0 && !yyTokColonSeen)
                 prospectiveContext = prefix;
             prefix += strColons;
             yyTok = getToken();
-#ifdef DIAGNOSE_RETRANSLATABILITY
-            if (yyTok == Tok_Ident && yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0) {
-                functionName = yyWord;
-                functionName.detach();
-            }
-#endif
             break;
         case Tok_RightBrace:
             if (!yyTokColonSeen) {
-                if (yyBraceDepth + 1 == namespaceDepths.count()) {
+                if (yyBraceDepth + 1 == namespaceDepths.size()) {
                     // class or namespace
                     truncateNamespaces(&namespaces, namespaceDepths.pop());
                 }
-                if (yyBraceDepth == namespaceDepths.count()) {
+                if (yyBraceDepth == namespaceDepths.size()) {
                     // function, class or namespace
                     if (!yyBraceDepth && !directInclude)
                         truncateNamespaces(&functionContext, 1);
@@ -2021,6 +2001,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             }
             Q_FALLTHROUGH();
         case Tok_Semicolon:
+            maybeInTrailingReturnType = false;
             prospectiveContext.clear();
             prefix.clear();
             if (!sourcetext.isEmpty() || !extracomment.isEmpty() || !msgid.isEmpty() || !extra.isEmpty()) {
@@ -2044,7 +2025,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             break;
         case Tok_Colon:
         case Tok_Equals:
-            if (yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0) {
+            if (yyBraceDepth == namespaceDepths.size() && yyParenDepth == 0) {
                 if (!prospectiveContext.isEmpty()) {
                     pendingContext = prospectiveContext;
                     prospectiveContext.clear();
@@ -2059,7 +2040,7 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
             yyTok = getToken();
             break;
         case Tok_LeftBrace:
-            if (yyBraceDepth == namespaceDepths.count() + 1 && yyParenDepth == 0) {
+            if (yyBraceDepth == namespaceDepths.size() + 1 && yyParenDepth == 0) {
                 if (!prospectiveContext.isEmpty()) {
                     pendingContext = prospectiveContext;
                     prospectiveContext.clear();
@@ -2069,24 +2050,61 @@ void CppParser::parseInternal(ConversionData &cd, const QStringList &includeStac
                     yyTokColonSeen = false;
                 }
             }
-            Q_FALLTHROUGH();
-        case Tok_LeftParen:
+            maybeInTrailingReturnType = false;
             yyTokIdentSeen = false;
-            Q_FALLTHROUGH();
+            metaExpected = true;
+            yyTok = getToken();
+            break;
+        case Tok_LeftParen:
+            if (!yyTokColonSeen && yyBraceDepth == namespaceDepths.size() && yyParenDepth == 1
+                && !prospectiveContext.isEmpty()) {
+                pendingContext = prospectiveContext;
+                prospectiveContext.clear();
+            }
+            yyTokIdentSeen = false;
+            metaExpected = true;
+            yyTok = getToken();
+            break;
         case Tok_Comma:
         case Tok_QuestionMark:
             metaExpected = true;
             yyTok = getToken();
             break;
         case Tok_RightParen:
-            if (yyParenDepth == 0)
+            if (yyParenDepth == 0) {
+                if (!yyTokColonSeen && !pendingContext.isEmpty()
+                    && yyBraceDepth == namespaceDepths.size()) {
+                    // Demote the pendingContext to prospectiveContext.
+                    prospectiveContext = pendingContext;
+                    pendingContext.clear();
+                }
                 metaExpected = true;
-            else
+            } else {
                 metaExpected = false;
+            }
             yyTok = getToken();
             break;
+        case Tok_decltype:
+            {
+                // Save the parentheses depth outside the 'decltype' specifier.
+                auto initialParenDepth = yyParenDepth;
+
+                // Eat the opening parenthesis that follows 'decltype'.
+                yyTok = getToken();
+
+                // Skip over everything within the parentheses that follow 'decltype'.
+                while (yyParenDepth != initialParenDepth && yyTok != Tok_Eof)
+                    yyTok = getToken();
+            }
+            break;
+        case Tok_enum:
+            yyTok = getToken();
+            // If it is an enum class then ignore
+            if (yyTok == Tok_class)
+                yyTok = getToken();
+            break;
         default:
-            if (!yyParenDepth)
+            if (!yyParenDepth && !maybeInTrailingReturnType)
                 prospectiveContext.clear();
             Q_FALLTHROUGH();
         case Tok_RightBracket: // ignoring indexing; for static initializers
@@ -2128,15 +2146,21 @@ void CppParser::processComment()
         yyWord.remove(0, 2);
         text = yyWord.trimmed();
         int k = text.indexOf(QLatin1Char(' '));
-        if (k > -1)
-            extra.insert(text.left(k), text.mid(k + 1).trimmed());
+        if (k > -1) {
+            QString commentvalue = text.mid(k + 1).trimmed();
+            if (commentvalue.startsWith(QLatin1Char('"')) && commentvalue.endsWith(QLatin1Char('"'))
+                && commentvalue.size() != 1) {
+                commentvalue = commentvalue.sliced(1, commentvalue.size() - 2);
+            }
+            extra.insert(text.left(k), commentvalue);
+        }
         text.clear();
     } else if (*ptr == QLatin1Char('%') && ptr[1].isSpace()) {
-        sourcetext.reserve(sourcetext.length() + yyWord.length() - 2);
-        ushort *ptr = (ushort *)sourcetext.data() + sourcetext.length();
+        sourcetext.reserve(sourcetext.size() + yyWord.size() - 2);
+        ushort *ptr = (ushort *)sourcetext.data() + sourcetext.size();
         int p = 2, c;
         forever {
-            if (p >= yyWord.length())
+            if (p >= yyWord.size())
                 break;
             c = yyWord.unicode()[p++].unicode();
             if (isspace(c))
@@ -2146,7 +2170,7 @@ void CppParser::processComment()
                 break;
             }
             forever {
-                if (p >= yyWord.length()) {
+                if (p >= yyWord.size()) {
                   whoops:
                     yyMsg() << "Unterminated meta string\n";
                     break;
@@ -2155,7 +2179,7 @@ void CppParser::processComment()
                 if (c == '"')
                     break;
                 if (c == '\\') {
-                    if (p >= yyWord.length())
+                    if (p >= yyWord.size())
                         goto whoops;
                     c = yyWord.unicode()[p++].unicode();
                     if (c == '\n')
@@ -2172,10 +2196,10 @@ void CppParser::processComment()
         ushort c;
         while ((c = uc[idx]) == ' ' || c == '\t' || c == '\n')
             ++idx;
-        if (!memcmp(uc + idx, MagicComment.unicode(), MagicComment.length() * 2)) {
-            idx += MagicComment.length();
+        if (!memcmp(uc + idx, CppMagicComment.unicode(), CppMagicComment.size() * 2)) {
+            idx += CppMagicComment.size();
             comment = QString::fromRawData(yyWord.unicode() + idx,
-                                           yyWord.length() - idx).simplified();
+                                           yyWord.size() - idx).simplified();
             int k = comment.indexOf(QLatin1Char(' '));
             if (k == -1) {
                 context = comment;
@@ -2209,7 +2233,7 @@ const ParseResults *CppParser::recordResults(bool isHeader)
     }
     if (isHeader) {
         const ParseResults *pr;
-        if (!tor && results->includes.count() == 1
+        if (!tor && results->includes.size() == 1
             && results->rootNamespace.children.isEmpty()
             && results->rootNamespace.aliases.isEmpty()
             && results->rootNamespace.usings.isEmpty()) {
@@ -2220,7 +2244,7 @@ const ParseResults *CppParser::recordResults(bool isHeader)
             results->fileId = nextFileId++;
             pr = results;
         }
-        CppFiles::setResults(yyFileName, pr);
+        CppFiles::setResults(ResultsCacheKey(yyFileName, *this), pr);
         return pr;
     } else {
         delete results;
@@ -2233,7 +2257,7 @@ void loadCPP(Translator &translator, const QStringList &filenames, ConversionDat
     QStringConverter::Encoding e = cd.m_sourceIsUtf16 ? QStringConverter::Utf16 : QStringConverter::Utf8;
 
     for (const QString &filename : filenames) {
-        if (!CppFiles::getResults(filename).isEmpty() || CppFiles::isBlacklisted(filename))
+        if (!CppFiles::getResults(ResultsCacheKey(filename)).isEmpty() || CppFiles::isBlacklisted(filename))
             continue;
 
         QFile file(filename);
